@@ -203,6 +203,7 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
         kwArgs.reverse()
 
     loadValue = code.stack[funcIndex]
+    funcName = loadValue.getName(codeSource.module)
     returnValue = Stack.makeFuncReturnValue(loadValue)
     if loadValue.isMethodCall(codeSource.classObject, cfg().methodArgName) :
         methodName = loadValue.data[1]
@@ -211,13 +212,29 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
             if m != None :
                 _checkFunctionArgs(code, m, 1, argCount, kwArgs, check_arg_count)
         except KeyError :
-            if cfg().callingAttribute :
+            sattr = None
+            cobj = codeSource.classObject
+            # FIXME: this is an awful hack, there's got to be a better way
+            #   im_class meaning changed in 2.2, not sure how best to do this
+            if utils.pythonVersion() >= utils.PYTHON_2_2 :
+                sattr = cobj.statics.get(methodName)
+                if sattr is not None :
+                    funcName = sattr.getName(cobj)
+            else :
+                sattr = getattr(cobj.classObject, methodName, None)
+                if sattr is not None :
+                    funcName = str(getattr(sattr, 'im_class', sattr))
+                    funcName = funcName + '.' + sattr.__name__
+
+            if sattr is None and cfg().callingAttribute :
                 code.addWarning(msgs.INVALID_METHOD % methodName)
+
     elif loadValue.type in (Stack.TYPE_ATTRIBUTE, Stack.TYPE_GLOBAL) and \
          type(loadValue.data) in (types.StringType, types.TupleType) :
         # apply(func, (args)), can't check # of args, so just return func
         if loadValue.data == 'apply' :
             loadValue = code.stack[funcIndex+1]
+            funcName = loadValue.getName(codeSource.module)
         else :
             if cfg().modifyDefaultValue and \
                type(loadValue.data) == types.TupleType :
@@ -266,8 +283,7 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
                     code.addWarning(msgs.USING_NONE_RETURN_VALUE % name)
 
     code.stack = code.stack[:funcIndex] + [ returnValue ]
-    if loadValue is not None :
-        code.functionsCalled[loadValue.getName(codeSource.module)] = loadValue
+    code.functionsCalled[funcName] = loadValue
 
 
 def _classHasAttribute(c, attr) :
@@ -275,7 +291,12 @@ def _classHasAttribute(c, attr) :
             hasattr(c.classObject, attr))
 
 def _checkClassAttribute(attr, c, code) :
-    if not _classHasAttribute(c, attr) and cfg().classAttrExists :
+    if _classHasAttribute(c, attr) :
+        try :
+            del c.memberRefs[attr]
+        except KeyError :
+            pass
+    elif cfg().classAttrExists :
         code.addWarning(msgs.INVALID_CLASS_ATTR % attr)
 
 def _checkModuleAttribute(attr, module, code, ref) :
@@ -362,7 +383,7 @@ def _handleImport(code, operand, module, main, fromName) :
                 err = 'from %s import %s' % (tmpFromName, operand)
                 err = msgs.MODULE_MEMBER_IMPORTED_AGAIN % err
             elif modline1 is not None :
-                if cfg().mixImport :
+                if cfg().mixImport and code.lastLineNum != modline1[1] :
                     err = msgs.MIX_IMPORT_AND_FROM_IMPORT % tmpFromName
             else :
                 err = msgs.MODULE_MEMBER_ALSO_STAR_IMPORTED % fromName
@@ -455,17 +476,23 @@ def _getConstant(code, module, data) :
 _UNCHECKABLE_FORMAT_STACK_TYPES = \
       (Stack.TYPE_UNKNOWN, Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,)
 
-def _getFormatWarnings(code, codeSource) :
+def _getFormatString(code, codeSource) :
     if len(code.stack) <= 1 :
-        return
+        return ''
 
     format = code.stack[-2]
     if format.type != types.StringType or not format.const :
         format = _getConstant(code, codeSource.module, format)
         if format is None or type(format) != types.StringType :
-            return
-    else :
-        format = format.data
+            return ''
+        return format
+    return format.data
+
+    
+def _getFormatWarnings(code, codeSource) :
+    format = _getFormatString(code, codeSource)
+    if not format :
+        return
 
     args = 0
     count, vars = _getFormatInfo(format, code)
@@ -538,14 +565,11 @@ class Code :
         self.lastLineNum = 0
         self.maxCode = 0
 
-        self.lastReturnLabel = 0
-        self.maxLabel = 0
-
         self.returnValues = []
+        self.raiseValues = []
         self.stack = []
 
         self.unpackCount = 0
-        self.returns = 0
         self.loops = 0
         self.branches = {}
 
@@ -575,18 +599,18 @@ class Code :
             line = self.lastLineNum
         self.warnings.append(Warning.Warning(self.func_code, line, err))
 
-    def getNextOp(self) :
+    def popNextOp(self) :
+        oldIndex = self.index
         info = OP.getInfo(self.bytes, self.index, self.extended_arg)
         op, oparg, self.index, self.extended_arg = info
         if op < OP.HAVE_ARGUMENT :
-            utils.debug("  " + str(self.index) + " " + OP.name[op])
+            utils.debug("  %d %s" % (oldIndex, OP.name[op]))
             operand = None
         else :
             operand = OP.getOperand(op, self.func_code, oparg)
             self.label = label = OP.getLabel(op, oparg, self.index)
-            utils.debug("  " + str(self.index) + " " + OP.name[op], oparg, operand)
+            utils.debug("  %d %s" % (oldIndex, OP.name[op]), oparg, operand)
             if label != None :
-                self.maxLabel = max(label, self.maxLabel)
                 if self.branches.has_key(label) :
                     self.branches[label] = self.branches[label] + 1
                 else :
@@ -595,8 +619,10 @@ class Code :
         return op, oparg, operand
 
     def nextOpInfo(self) :
-        info = OP.getInfo(self.bytes, self.index, 0)
-        return info[0:2]
+        try :
+            return OP.getInfo(self.bytes, self.index, 0)[0:3]
+        except IndexError :
+            return -1, 0, -1
 
     def getFirstOp(self) :
         # find the first real op, maybe we should not check if params are used
@@ -655,13 +681,21 @@ class Code :
         self.typeMap[name] = valueList
             
     def addReturn(self) :
-        self.returns = self.returns + 1
-        self.lastReturnLabel = self.index - utils.BACK_RETURN_INDEX
         if len(self.stack) > 0 :
-            self.returnValues.append((self.lastLineNum, self.stack[-1]))
+            value = (self.lastLineNum, self.stack[-1], self.nextOpInfo()[2])
+            self.returnValues.append(value)
             self.popStack()
 
+    def addRaise(self) :
+        self.raiseValues.append((self.lastLineNum, None, self.index))
+
     def removeBranch(self, label) :
+        # FIXME: not sure what to do here, we need all the branches
+        #        for checking unused code (I think)
+        #        but it's an extra branch for code complexity,
+        #        right now, unused code is more important
+        return
+
         branch = self.branches.get(label, None)
         if branch is not None :
             if branch == 1 :
@@ -709,6 +743,9 @@ def _STORE_NAME(oparg, operand, codeSource, code) :
         if not codeSource.in_class :
             _checkGlobal(operand, module, codeSource.func, code,
                          msgs.GLOBAL_DEFINED_NOT_DECLARED, codeSource.main)
+        else :
+            if code.stack :
+                codeSource.classObject.statics[operand] = code.stack[-1]
 
         var = module.variables.get(operand)
         if var is not None and code.stack and code.stack[-1].const :
@@ -732,7 +769,7 @@ def _checkLoadGlobal(codeSource, code, varname) :
         if not codeSource.main and \
            hasattr(codeSource.calling_code, 'function') and \
            varname in codeSource.calling_code.function.func_code.co_varnames :
-            _handleLoadLocal(code, codeSource.func, varname)
+            _handleLoadLocal(code, codeSource, varname)
             should_check = 0
 
     if should_check :
@@ -744,6 +781,8 @@ def _checkLoadGlobal(codeSource, code, varname) :
 
 def _LOAD_NAME(oparg, operand, codeSource, code) :
     _checkLoadGlobal(codeSource, code, operand)
+    if OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
     # if there was from XXX import *, _* names aren't imported
     if codeSource.module.modules.has_key(operand) and \
@@ -761,11 +800,15 @@ _LOAD_GLOBAL = _LOAD_NAME
 
 def _LOAD_DEREF(oparg, operand, codeSource, code) :
     if type(oparg) == types.IntType :
-        argused = code.func_code.co_varnames[oparg]
-        code.unusedLocals[argused] = None
+        if code.func_code.co_name != utils.LAMBDA :
+            argused = code.func_code.co_varnames[oparg]
+            code.unusedLocals[argused] = None
+        if OP.POP_TOP(code.nextOpInfo()[0]) :
+            code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
     else :
         _LOAD_GLOBAL(oparg, operand, codeSource, code)
 
+_LOAD_CLOSURE = _LOAD_DEREF
 
 def _DELETE_NAME(oparg, operand, codeSource, code) :
     _checkLoadGlobal(codeSource, code, operand)
@@ -784,24 +827,36 @@ def _LOAD_CONST(oparg, operand, codeSource, code) :
             code.codeObjects[name] = operand
         elif cfg().redefiningFunction :
             code.addWarning(msgs.REDEFINING_ATTR % (name, obj.co_firstlineno))
+    elif OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
 
-def _checkLoadLocal(code, func, varname, deletedWarn, usedBeforeSetWarn) :
+def _checkLocalShadow(code, module, varname) :
+    if module.variables.has_key(varname) :
+        line = module.moduleLineNums.get(varname, (0, 0))[1]
+        if cfg().shadows :
+            code.addWarning(msgs.LOCAL_SHADOWS_GLOBAL % (varname, line))
+
+def _checkLoadLocal(code, codeSource, varname, deletedWarn, usedBeforeSetWarn) :
     _checkFutureKeywords(code, varname)
     deletedLine = code.deletedLocals.get(varname)
     if deletedLine :
         code.addWarning(deletedWarn % (varname, deletedLine))
-    elif not code.unusedLocals.has_key(varname) and not func.isParam(varname) :
+    elif not code.unusedLocals.has_key(varname) and \
+         not codeSource.func.isParam(varname) :
         code.addWarning(usedBeforeSetWarn % varname)
     code.unusedLocals[varname] = None
+    _checkLocalShadow(code, codeSource.module, varname)
 
-def _handleLoadLocal(code, func, varname) :
-    _checkLoadLocal(code, func, varname,
+def _handleLoadLocal(code, codeSource, varname) :
+    _checkLoadLocal(code, codeSource, varname,
                     msgs.LOCAL_DELETED, msgs.VAR_USED_BEFORE_SET)
 
 def _LOAD_FAST(oparg, operand, codeSource, code) :
     code.stack.append(Stack.Item(operand, type(operand)))
-    _handleLoadLocal(code, codeSource.func, operand)
+    _handleLoadLocal(code, codeSource, operand)
+    if OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
 def _STORE_FAST(oparg, operand, codeSource, code) :
     if not code.updateCheckerArgs(operand) :
@@ -814,6 +869,7 @@ def _STORE_FAST(oparg, operand, codeSource, code) :
             else :
                 code.constants[operand] = code.stack[-1].data
 
+        _checkLocalShadow(code, codeSource.module, operand)
         _checkException(code, operand)
         if code.deletedLocals.has_key(operand) :
             del code.deletedLocals[operand]
@@ -825,15 +881,14 @@ def _STORE_FAST(oparg, operand, codeSource, code) :
         code.unpack()
 
 def _DELETE_FAST(oparg, operand, codeSource, code) :
-    _checkLoadLocal(code, codeSource.func, operand,
+    _checkLoadLocal(code, codeSource, operand,
                     msgs.LOCAL_ALREADY_DELETED, msgs.VAR_DELETED_BEFORE_SET)
     code.deletedLocals[operand] = code.lastLineNum
 
 def _checkAttribute(top, operand, codeSource, code) :
     if top.data == cfg().methodArgName and codeSource.classObject != None :
         _checkClassAttribute(operand, codeSource.classObject, code)
-    elif type(top.type) == types.StringType or \
-         top.type == types.ModuleType :
+    elif type(top.type) == types.StringType or top.type == types.ModuleType :
         _checkModuleAttribute(operand, codeSource.module, code, top.data)
     else :
         _checkAttributeType(code, top, operand)
@@ -861,8 +916,12 @@ def _LOAD_ATTR(oparg, operand, codeSource, code) :
         top = code.stack[-1]
         _checkAttribute(top, operand, codeSource, code)
         top.addAttribute(operand)
-        if not OP.LOAD_ATTR(code.nextOpInfo()[0]) :
-            _checkExcessiveReferences(code, top)
+        nextOp = code.nextOpInfo()[0]
+        if not OP.LOAD_ATTR(nextOp) :
+            if OP.POP_TOP(nextOp) :
+                code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+            else :
+                _checkExcessiveReferences(code, top)
 
 def _STORE_ATTR(oparg, operand, codeSource, code) :
     _checkExcessiveReferences(code, code.stack[-1], operand)
@@ -881,7 +940,7 @@ def _IMPORT_NAME(oparg, operand, codeSource, code) :
     if not OP.IMPORT_FROM(nextOp) and not OP.IMPORT_STAR(nextOp) :
         # handle import xml.sax as sax
         if OP.LOAD_ATTR(nextOp) :
-            operand = code.getNextOp()[2]
+            operand = code.popNextOp()[2]
         _handleImport(code, operand, codeSource.module, codeSource.main, None)
 
 def _IMPORT_FROM(oparg, operand, codeSource, code) :
@@ -924,6 +983,7 @@ def _CALL_FUNCTION_VAR_KW(oparg, operand, codeSource, code) :
 
 def _MAKE_FUNCTION(oparg, operand, codeSource, code) :
     code.popStackItems(oparg+1)
+_MAKE_CLOSURE = _MAKE_FUNCTION
 
 def _BUILD_MAP(oparg, operand, codeSource, code) :
     _makeConstant(code.stack, oparg, Stack.makeDict)
@@ -945,7 +1005,7 @@ def _UNARY_CONVERT(oparg, operand, codeSource, code) :
 def _UNARY_POSITIVE(oparg, operand, codeSource, code) :
     if OP.UNARY_POSITIVE(code.nextOpInfo()[0]) :
         code.addWarning(msgs.STMT_WITH_NO_EFFECT % '++')
-        code.getNextOp()
+        code.popNextOp()
     elif cfg().unaryPositive and code.stack and not code.stack[-1].const :
         code.addWarning(msgs.UNARY_POSITIVE_HAS_NO_EFFECT)
 
@@ -964,8 +1024,8 @@ def _popStackRef(code, operand, count = 2) :
 
 def _pop(oparg, operand, codeSource, code) :
     code.popStack()
-_POP_TOP = _BINARY_POWER = _BINARY_MULTIPLY = _BINARY_DIVIDE = \
-           _BINARY_SUBTRACT = _BINARY_LSHIFT = _BINARY_RSHIFT = \
+_POP_TOP = _BINARY_POWER = _BINARY_SUBTRACT = \
+           _BINARY_LSHIFT = _BINARY_RSHIFT = \
            _BINARY_AND = _BINARY_XOR = _BINARY_OR = _pop
 
 def _BINARY_ADD(oparg, operand, codeSource, code) :
@@ -1001,6 +1061,14 @@ def _BINARY_DIVIDE(oparg, operand, codeSource, code) :
 
     code.popStack()
 
+def _BINARY_MULTIPLY(oparg, operand, codeSource, code) :
+    if len(code.stack) >= 2 :
+        format = _getFormatString(code, codeSource)
+        if format and type(code.stack[-1].data) == types.IntType :
+            code.stack[-2].data = format * code.stack[-1].data
+
+    code.popStack()
+
 def _BINARY_MODULO(oparg, operand, codeSource, code) :
     _getFormatWarnings(code, codeSource)
     code.popStack()
@@ -1038,7 +1106,15 @@ def _jump(oparg, operand, codeSource, code) :
             name = topOfStack.data[-1]
             if codeSource.classObject.methods.has_key(name) :
                 code.addWarning(msgs.USING_METHOD_AS_ATTR % name)
-_JUMP_IF_FALSE = _JUMP_IF_TRUE = _JUMP_ABSOLUTE = _jump
+_JUMP_ABSOLUTE = _jump
+
+def _jump_conditional(oparg, operand, codeSource, code) :
+    if code.stack and code.stack[-1].const :
+        if cfg().constantConditions and \
+           (code.stack[-1].data != 1 or cfg().constant1) :
+            code.addWarning(msgs.CONSTANT_CONDITION % str(code.stack[-1]))
+    _jump(oparg, operand, codeSource, code)
+_JUMP_IF_FALSE = _JUMP_IF_TRUE = _jump_conditional
 
 
 def _JUMP_FORWARD(oparg, operand, codeSource, code) :
@@ -1052,6 +1128,9 @@ def _RETURN_VALUE(oparg, operand, codeSource, code) :
     if codeSource.calling_code is None :
         code.addReturn()
 
+def _RAISE_VARARGS(oparg, operand, codeSource, code) :
+    code.addRaise()
+
 
 DISPATCH = [ None ] * 256
 DISPATCH[  1] = _POP_TOP
@@ -1107,8 +1186,11 @@ DISPATCH[124] = _LOAD_FAST
 DISPATCH[125] = _STORE_FAST
 DISPATCH[126] = _DELETE_FAST
 DISPATCH[127] = _LINE_NUM
+DISPATCH[130] = _RAISE_VARARGS
 DISPATCH[131] = _CALL_FUNCTION
 DISPATCH[132] = _MAKE_FUNCTION
+DISPATCH[134] = _MAKE_CLOSURE
+DISPATCH[135] = _LOAD_CLOSURE
 DISPATCH[136] = _LOAD_DEREF
 DISPATCH[140] = _CALL_FUNCTION_VAR
 DISPATCH[141] = _CALL_FUNCTION_KW
