@@ -14,7 +14,7 @@ import re
 
 _RC_FILE = ".pycheckrc"
 CHECKER_VAR = '__pychecker__'
-_VERSION = '0.8.6'
+_VERSION = '0.8.7'
 
 _DEFAULT_BLACK_LIST = [ "Tkinter", "wxPython", "gtk", "GTK", "GDK", ]
 _DEFAULT_VARIABLE_IGNORE_LIST = [ '__version__', '__warningregistry__', 
@@ -37,6 +37,7 @@ _OPTIONS = (
  ('x', 0, 'miximport', 'mixImport', 'module does import and from ... import'),
  ('l', 0, 'local', 'localVariablesUsed', 'unused local variables, except tuples'),
  ('t', 0, 'tuple', 'unusedLocalTuple', 'all unused local variables, including tuples'),
+ ('9', 0, 'members', 'membersUsed', 'all unused class data members'),
  ('v', 0, 'var', 'allVariablesUsed', 'all unused module variables'),
  ('p', 0, 'privatevar', 'privateVariableUsed', 'unused private module variables'),
  ('g', 0, 'allglobals', 'reportAllGlobals', 'report each occurrence of global warnings'),
@@ -45,6 +46,9 @@ _OPTIONS = (
  ('I', 0, 'initsubclass', 'initDefinedInSubclass', 'Subclass.__init__() not defined'),
  ('u', 0, 'callinit', 'baseClassInitted', 'Subclass.__init__() not called'),
  ('N', 0, 'initreturn', 'returnNoneFromInit', 'Return None from __init__()'),
+ ('8', 0, 'unreachable', 'unreachableCode', 'unreachable code'),
+ ('2', 0, 'constCond', 'constantConditions', 'a constant is used in a conditional statement'),
+ ('1', 0, 'constant1', 'constant1', '1 is used in a conditional statement (if 1: or while 1:)'),
  ('A', 0, 'callattr', 'callingAttribute', 'Calling data members as functions'),
  ('y', 0, 'classattr', 'classAttrExists', 'class attribute does not exist'),
  ('S', 1, 'self', 'methodArgName', 'First argument to methods'),
@@ -61,6 +65,7 @@ _OPTIONS = (
  ('C', 0, 'implicitreturns', 'checkImplicitReturns', 'check if using implict and explicit return values'),
  ('O', 0, 'objattrs', 'checkObjectAttrs', 'check that attributes of objects exist'),
  ('D', 0, 'intdivide', 'intDivide', 'check if using integer division'),
+ ('w', 0, 'shadow', 'shadows', 'check if local variable shadows a global'),
      ]),
     ('Suppressions', [
  ('q', 0, 'stdlib', 'ignoreStandardLibrary', 'ignore warnings from files under standard library'),
@@ -176,6 +181,7 @@ class Config :
         self.reportAllGlobals = 0
         self.allVariablesUsed = 0
         self.privateVariableUsed = 1
+        self.membersUsed = 0
         self.importUsed = 1
         self.reimportSelf = 1
         self.moduleImportErrors = 1
@@ -189,6 +195,9 @@ class Config :
         self.classAttrExists = 1
         self.namedArgs = 1
         self.returnNoneFromInit = 1
+        self.unreachableCode = 0
+        self.constantConditions = 1
+        self.constant1 = 0
 
         self.unusedNames = _DEFAULT_UNUSED_LIST
         self.variablesToIgnore = _DEFAULT_VARIABLE_IGNORE_LIST
@@ -211,8 +220,9 @@ class Config :
 
         self.checkObjectAttrs = 1
         self.checkReturnValues = 1
-        self.checkImplicitReturns = 0
+        self.checkImplicitReturns = 1
         self.intDivide = 1
+        self.shadows = 1
         self.unaryPositive = 1
         self.modifyDefaultValue = 1
 
