@@ -30,11 +30,22 @@ class Item :
         self.length = length
 
     def __str__(self) :
+        if type(self.data) == types.TupleType :
+            value = '('
+            for item in self.data :
+                value = value + str(item) + ', '
+            # strip off the ', ' for mulitple items
+            if len(self.data) > 1 :
+                value = value[:-2]
+            return value + ')'
         return str(self.data)
 
     def isNone(self) :
         return self.data is None or (self.data == 'None' and not self.const)
 
+    def isImplicitNone(self) :
+        return self.data is None and self.const
+
     def isMethodCall(self, c, methodArgName) :
         return self.type == TYPE_ATTRIBUTE and c != None and \
                len(self.data) == 2 and self.data[0] == methodArgName
