@@ -178,6 +178,8 @@ class Class :
         self.members = { '__class__': types.ClassType,
                          '__doc__': types.StringType,
                          '__dict__': types.DictType, }
+        self.memberRefs = {}
+        self.statics = {}
 
     def getFirstLine(self) :
         "Return first line we can find in THIS class, not any base classes"
@@ -226,10 +228,12 @@ class Class :
         for classToken in _getClassTokens(classObject) :
             token = getattr(classObject, classToken)
             if type(token) == types.MethodType :
-                self.addMethod(token.im_func, classObject.__name__, classToken)
+                self.addMethod(token.im_func, classObject.__name__, token.__name__)
             else :
                 self.members[classToken] = type(token)
+                self.memberRefs[classToken] = None
 
+        self.cleanupMemberRefs()
         # add standard methods
         for methodName in ('__class__',) :
             self.addMethod(methodName, classObject.__name__)
@@ -262,8 +266,16 @@ class Class :
                             if len(stack) > 1 :
                                 value = type(stack[-2])
                             self.members[operand] = value
+                            self.memberRefs[operand] = None
                         stack = []
 
+        self.cleanupMemberRefs()
+
+    def cleanupMemberRefs(self) :
+        try :
+            del self.memberRefs['__pychecker__']
+        except KeyError :
+            pass
 
 def importError(moduleName, info):
     # detail may contain a newline replace with - 
@@ -324,7 +336,10 @@ class Module :
     def filename(self) :
         if not self.module :
             return self.moduleName
-        filename = self.module.__file__
+        try :
+            filename = self.module.__file__
+        except AttributeError :
+            filename = self.moduleName
         if string.lower(filename[-4:]) == '.pyc' :
             filename = filename[:-4] + '.py'
         return filename
