@@ -17,6 +17,7 @@ VAR_NOT_USED = "Variable (%s) not used"
 IMPORT_NOT_USED = "Imported module (%s) not used"
 UNUSED_LOCAL = "Local variable (%s) not used"
 UNUSED_PARAMETER = "Parameter (%s) not used"
+UNUSED_MEMBERS = "Members (%s) not used in class (%s)"
 NO_LOCAL_VAR = "No local variable (%s)"
 VAR_USED_BEFORE_SET = "Variable (%s) used before being set"
 
@@ -39,6 +40,8 @@ INVALID_GLOBAL = "No global (%s) found"
 INVALID_METHOD = "No method (%s) found"
 INVALID_CLASS_ATTR = "No class attribute (%s) found"
 INVALID_MODULE_ATTR = "No module attribute (%s) found"
+
+LOCAL_SHADOWS_GLOBAL = "Local variable (%s) shadows global defined on line %d"
 USING_METHOD_AS_ATTR = "Using method (%s) as an attribute (not invoked)"
 OBJECT_HAS_NO_ATTR = "Object (%s) has no attribute (%s)"
 METHOD_SIGNATURE_MISMATCH = "Overridden method (%s) doesn't match signature in class (%s)"
@@ -61,6 +64,8 @@ TOO_MANY_REFERENCES = 'Law of Demeter violated, more than %d references for (%s)
 
 IMPLICIT_AND_EXPLICIT_RETURNS = "Function returns a value and also implicitly returns None"
 INCONSISTENT_RETURN_TYPE = "Function return types are inconsistent"
+CODE_UNREACHABLE = "Code appears to be unreachable"
+CONSTANT_CONDITION = "Using a conditional statement with a constant value (%s)"
 
 INVALID_FORMAT = "Invalid format string, problem starts near: '%s'"
 INVALID_FORMAT_COUNT = "Format string argument count (%d) doesn't match arguments (%d)"
@@ -72,6 +77,7 @@ INTEGER_DIVISION = "Using integer division (%s / %s) may return integer or float
 USING_TUPLE_ACCESS_TO_LIST = "Using a tuple instead of slice as list accessor for (%s)"
 
 STMT_WITH_NO_EFFECT = "Operator (%s) doesn't exist, statement has no effect"
+POSSIBLE_STMT_WITH_NO_EFFECT = "Statement appears to have no effect"
 UNARY_POSITIVE_HAS_NO_EFFECT = "Unary positive (+) usually has no effect"
 LIST_APPEND_ARGS = "[].append() only takes 1 argument in Python 1.6 and above for (%s)"
 
